      subroutine define_arma(levelmax,arma,maxnumdist,maxref,numdist,
     *   numarma,distdim,distyp,distn,distpos,distrestrpos,fulltype,
     *   resname,distvar,ifull,resref,line,il,ipt,jpt)
 
c     Subroutine to define an ARMA distribution.  The syntax is:
c
c     DEFINE {ar(p) | ma(q) | arma(p,q) | car(1)} DISTRIBUTION;  [DIM = 1;]
c        [ARDOMAIN = {(-1,1) | (0,1)};]
c        [MADOMAIN = {(-1,1) | (0,1) | (0,Inf)};]
c        TIMEVAR [(WITHIN LEVEL n)] = varname;  [INCREASING={yes|no};]
c        NAME = [name]; [REF = ...;]
c
c     By default, the domain restrictions for autoregressive and moving
c     average parameters is (-1,1).  Note that this is a necessary
c     condition only!  Optionally, the domains may be restricted to (0,1).
c     Note that ARMA(p,q) with p>1 requires that the time variable
c     always takes on integer values.  A C/AR(1) may have non-integer
c     values, provided ARDOMAIN=(0,1).
c
c     The starting values of ARMA(p,q) processes are:  first p phi's,
c     then variance of innovation eps, then q theta's.
c
c     Note: for arma processes,
c        distyp = 1000*l + 100*p + 10*q + 2
c        where l = level across which autocorrelation is present
c              p = autoregressive order
c              q = moving average order
c     so that  l = distyp/1000
c              p = mod(distyp,1000)/100
c              q = mod(distyp,100)/10
c
c     and for cumulative arma processes:
c        distyp = 1000*l + 100*p + 10*q + 3
c     but only CAR(1) is currently supported, so distyp = 1000*l + 103.
c
c     NOTE:  by default, time variables must be strictly increasing
c     within their specified level.  This check may be turned off,
c     which is recorded by making distyp negative.
c
c     The maximum autoregressive and moving average orders are hardwired
c     to 9.  Increasing this would involve changing the coding of distyp
c     throughout all routines.
c
c     Constantijn Panis, 1 February 2000
c     Last change: reference numbers
 
      implicit none
      integer levelmax,arma,maxnumdist,maxref,numdist,numarma,
     *   distdim(maxnumdist),distn(maxnumdist),distyp(maxnumdist),
     *   distpos(maxnumdist),distrestrpos(maxnumdist),fulltype(*),
     *   ifull,resref(maxref),beginref,numname,nref,dim,flag,
     *   l,p,q,ardomain,madomain,il,i1,i2,ipt,jpt,status,intvecsum
      logical smtxt,dupchar12
      character line*128,resname(*)*12,distvar(maxnumdist)*8
 
      numdist=numdist+1
      numarma=numarma+1
      if(numdist.gt.maxnumdist)call bug(ipt,jpt)
      distdim(numdist)=1
      distvar(numdist)='????????'
c     Defaults:  -1<phi<1, no restriction on theta.
      ardomain=2
      madomain=0
      p=0
      q=0
c     Zero means that there are no restrictions:
      distrestrpos(numdist)=0
      distpos(numdist)=ifull+1
      numname=intvecsum(distdim,numdist-1)
c     Locate begin point of distribution reference numbers:
      call find_br(-1,numdist,maxref,resref,beginref,ipt,jpt)
 
c     DEFINE {AR(p) | MA(q) | ARMA(p,q) | CAR(1)} DISTRIBUTION;
      if(arma.eq.1)then
         call getint(4,p,line,il,status)
         if(status.gt.0)goto 800
      elseif(arma.eq.2)then
         call getint(4,q,line,il,status)
         if(status.gt.0)goto 800
      elseif(arma.eq.3)then
         call getint(4,p,line,il,status)
         if(status.gt.0)goto 800
         if(.not.smtxt(4,line,il,','))goto 800
         call getint(4,q,line,il,status)
         if(status.gt.0)goto 800
      elseif(arma.eq.11)then
         call getint(4,p,line,il,status)
         if(status.gt.0)goto 800
      else
         call bug(ipt,jpt)
      endif
      if(.not.smtxt(4,line,il,') DISTribution;'))goto 800
      if(p.ge.10.or.q.ge.10)goto 830
      if(arma.lt.10)then
         distyp(numdist)=100*p+10*q+2
      else
c        Only CAR(1) is currently supported:
         if(p.ne.1.or.q.ne.0)goto 850
         distyp(numdist)=100*p+10*q+3
      endif
      distn(numdist)=p+1+q
 
  100 if(smtxt(4,line,il,'DIMension='))goto 110
      if(smtxt(4,line,il,'TIMEVAR'))goto 150
      if(smtxt(4,line,il,'ARDOMAIN ='))goto 170
      if(smtxt(4,line,il,'MADOMAIN ='))goto 180
      if(smtxt(4,line,il,'NAME ='))goto 190
 
c     Nothing matches.  Either end of definition or syntax error.
c     Check that names or reference numbers have been assigned:
      if(numname.ne.intvecsum(distdim,numdist))goto 800
c     Finish reference numbers with a -1, even if none are defined:
      resref(beginref)=-1
c     Check that arvar has been specified:
      if(distvar(numdist).eq.'????????')goto 800
c     Order of starting values is phi, vareps, theta:
      if(p.gt.0)call putint(ardomain,fulltype(ifull+1),p)
      fulltype(ifull+p+1)=1
      if(q.gt.0)call putint(madomain,fulltype(ifull+p+2),q)
c     Keep track of total number of parameters defined thus far:
      ifull=ifull+distn(numdist)
      goto 999
 
c     DIM = n;
  110 call getint(4,dim,line,il,status)
      if(status.gt.0)goto 800
      if(.not.smtxt(4,line,il,';'))goto 800
      if(dim.ne.1)goto 800
      goto 100
 
c     TIMEVAR [(WITHIN LEVEL n)] = varname;
  150 if(smtxt(4,line,il,'='))then
c        Default:  residuals all autocorrelated (identical to WITHIN LEVEL 1)
         distyp(numdist)=1000+distyp(numdist)
      elseif(smtxt(4,line,il,'('))then
         if(.not.smtxt(4,line,il,'WITHIN'))goto 800
         if(.not.smtxt(4,line,il,'LEVEL'))goto 800
         call getint(4,l,line,il,status)
         if(status.gt.0)goto 800
         if(l.le.0.or.l.gt.levelmax-1)goto 840
         if(.not.smtxt(4,line,il,') ='))goto 800
         distyp(numdist)=1000*l+distyp(numdist)
      else
         goto 800
      endif
      call getword(4,line,il,i1,i2,status)
      if(status.gt.0)goto 800
      if(i2-i1.ge.8)goto 800
      distvar(numdist)=line(i1:i2)
      if(.not.smtxt(4,line,il,';'))goto 800
 
c     [INCREASING [= {YES|NO}];]      (default:  yes)
      if(.not.smtxt(4,line,il,'INCReasing'))goto 100
      call semiyesno(flag,4,line,il,ipt,jpt)
      if(flag.eq.0)distyp(numdist)=-distyp(numdist)
      goto 100
 
c     ARDOMAIN = (-1,1) | (0,1);
  170 if(ardomain.ne.2)goto 800
      if(smtxt(4,line,il,'(-1,1);'))then
         ardomain=2
      elseif(smtxt(4,line,il,'(0,1);'))then
         ardomain=3
      else
         goto 800
      endif
      goto 100
 
c     MADOMAIN = (-1,1) | (0,1) | (0,Inf);
  180 if(madomain.ne.0)goto 800
      if(smtxt(4,line,il,'(-1,1);'))then
         madomain=2
      elseif(smtxt(4,line,il,'(0,1);'))then
         madomain=3
      elseif(smtxt(4,line,il,'(0,Inf);'))then
         madomain=1
      elseif(smtxt(4,line,il,'(0,+Inf);'))then
         madomain=1
      else
         goto 800
      endif
      goto 100
 
c     NAME = [name]; [REF = ...;]
  190 numname=numname+1
      if(smtxt(4,line,il,';'))then
         resname(numname)='(noname)'
      else
         call getword(4,line,il,i1,i2,status)
         if(status.gt.0)goto 800
         resname(numname)=line(i1:i2)
c        Check that name is unique:
         if(dupchar12(resname(numname),resname,numname-1))goto 820
         if(.not.smtxt(4,line,il,';'))goto 800
      endif
      nref=0
      if(smtxt(4,line,il,'REFerence='))then
         call readrefs(maxref,beginref,resref,nref,line,il,ipt,jpt)
      else
         if(resname(numname).eq.'(unnamed)')goto 800
         resref(beginref)=0
         beginref=beginref+1
      endif
      goto 100
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         if(arma.lt.10)write(8,801)
         if(arma.ge.10)write(8,802)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(arma.lt.10)write(6,801)
         if(arma.ge.10)write(6,802)
      endif
  801 format(/,'ERROR encountered in the definition of an ',
     *   'ARMA distribution.',/,'The syntax is:',/,
     *   3x,'DEFINE ARMA(p,q) DISTRIBUTION;  [DIM = 1;]',/,
     *   6x,'[ARDOMAIN = {(-1,1) | (0,1)};]',/,
     *   6x,'[MADOMAIN = {(-1,1) | (0,1)};]',/,
     *   6x,'TIMEVAR [(WITHIN LEVEL n)] = varname; ',
     *   '[INCREASING={yes|no};]',/,
     *   6x,'NAME = [name]; [REF = ...;]')
  802 format(/,'ERROR encountered in the definition of a Cumulative ',
     *   'AR(1) distribution.',/,'The syntax is:',/,
     *   3x,'DEFINE CAR(1) DISTRIBUTION;  [DIM = 1;]',/,
     *   6x,'[ARDOMAIN = {(-1,1) | (0,1)};]',/,
     *   6x,'TIMEVAR [(WITHIN LEVEL n)] = varname; ',
     *   '[INCREASING={yes|no};]',/,
     *   6x,'NAME = [name]; [REF = ...;]')
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format('ERROR in an ARMA/CAR(1) DISTRIBUTION definition.',/,
     *   'You assigned duplicate residual names.')
      stop
  830 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,831)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,831)
      endif
  831 format('ERROR:  the maximum autoregressive and moving average ',
     *   'orders are both 9.')
      stop
  840 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,841)
         call writea2ia(8,72,1,'number of levels in your data is ',
     *      levelmax,', so 1<=n<=',levelmax-1,'.  The default is '//
     *      '(WITHIN LEVEL 1).')
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,841)
         call writea2ia(6,72,1,'number of levels in your data is ',
     *      levelmax,', so 1<=n<=',levelmax-1,'.  The default is '//
     *      '(WITHIN LEVEL 1).')
      endif
  841 format('ERROR encountered in the time variable specification of ',
     *   'an ARMA',/,'distribution definition.  The syntax is:',/,
     *   6x,'TIMEVAR [(WITHIN LEVEL n)] = varname;',/,'where n is the ',
     *   'level across which there is no autocorrelation.',/,
     *   'Residuals are only autocorrelated at levels lower than n.  ',
     *   'The maximum')
      stop
  850 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,851)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,851)
      endif
  851 format('ERROR:  only the Cumulative AR(1) is supported.')
      stop
  999 return
      end
