      subroutine definv(type,a,n,status)
 
c     Subroutine to invert positive/negative definite matrix a(n,n).
c     The inverse will be stored in the same space as the original
c     matrix.  Only the lower triangle of a is used, but upon return
c     the entire matrix is filled with the inverse.
c     type = -1 if the matrix a is definite negative
c     type =  1 if the matrix a is definite positive
c
c     There is no check that the matrix is definite, but the results
c     of this routine crucially depend on that feature.
c
c     Note the results of this routine in cases where the matrix a is
c     less than full rank, i.e., semi-definite.  Suppose that the k-th
c     dimension is a linear combination of the i-th and j-th dimensions.
c     The "inverse" of a will then contain zeroes in the k-th column.
c     The product of the original matrix and its "inverse" will then also
c     contain zeroes in the k-th column, and the k-th row will contain
c     the linear combination factors that make up the k-th dimension
c     from the other dimensions.  This feature makes this routine very
c     attractive for computing the inverse of the information matrix,
c     followed by post-multiplication by the gradient to obtain the
c     serach direction.  If the information matrix is less than full
c     rank, i.e., the model is underidentified, then the search
c     direction in the redundant parameters will be zero, and by
c     analyzing the matrix times its "inverse" one can figure out
c     which parameters are responsible for the singularity.
c
c     Be very careful when attempting to modify this routine.
c     Every attempt is made to be as economical as possible with
c     storage space -- the upper triangle of a is used to store
c     intermediate results.  The diagonal of a is used just before
c     it is destroyed (subroutine yfactg) and the diagonal of the
c     inverse is computed right after it is not needed anymore for
c     intermediate results (subroutine t2ymlt).
c
c     Note the existence of subroutine nrminv, which will invert
c     any arbitrary square matrix, not just definite matrices.
c
c     Output integer argument status is zero if no error was detected.
c     The only error this routine checks for is whether the square
c     root is taken of a negative number.  If so, status=1.
c
c     Constantijn Panis,  16 May 2004 (previous version 27 October 1991)
c     Last change: include status argument.
 
      implicit none
      integer type,status,n
      double precision a(n,n)
 
      status=0
 
      if(n.eq.1)then
         if(a(1,1).ne.0.d0)a(1,1) = 1.d0/a(1,1)
         goto 999
      endif
 
      if(type.eq.-1)call opposite(a,a,n*n)
      call yinvg(n,a,a,status)
      if(status.ne.0)goto 999
      call reflect(a,n)
      if(type.eq.-1)call opposite(a,a,n*n)
 
  999 return
      end
c====================================================================
      subroutine yinvg(n,a,s,status)
      implicit none
      integer n,status
      double precision a(n,n),s(n,n)
 
      call yfactg(n,a,s,status)
      if(status.ne.0)goto 999
      call tinvg(n,s)
      call t2ymlt(n,s,a)
 
  999 return
      end
c====================================================================
      subroutine yfactg(n,a,s,status)
      implicit none
      integer n,status,i,j,ni
      double precision a(n,n),s(n,n),ss,ssss,aa,adiag,ratio,tol,innerp
      parameter(tol=1.d-15)
 
      do 100 i=1,n
         ni = i-1
         ss = innerp(s(1,i),s(1,i),ni)
         adiag = a(i,i)
         s(i,i) = 0.d0
         aa = 0.d0
         ss = adiag - ss
         ratio = ss / (adiag + 1.d-25)
         if(ratio.ge.tol)then
c           The following statement serves to eliminate a bug in the f77
c           compiler at Cornell (nero.ciser.cornell.edu, /usr/bin/f77):
c           if(i.eq.i+1)write(8,*)'Error in yfactg',i
            if(ss.lt.0.d0)goto 800
            aa = dsqrt(ss)
            s(i,i) = aa
            aa = 1.d0 / aa
         endif
         ss = aa
         do 110 j=i+1,n
            ssss = innerp(s(1,i),s(1,j),ni)
            s(i,j) = (a(j,i) - ssss ) * ss
  110 continue
  100 continue
      goto 999
  800 status=1
  999 return
      end
c====================================================================
      subroutine tinvg(n,x)
      implicit none
      integer n,i,j
      double precision x(n,n),small,ss,prod,xx
      parameter(small=1.d-25)
 
      do 100 i=1,n
         xx = x(i,i)
         ss = 0.d0
         if(xx.gt.small)ss = -1.d0/xx
         xx = ss
         do 110 j=1,i-1
            call inprod(i-j,n,1,x(j,j),x(j,i),prod)
            x(j,i) = prod*xx
  110    continue
         x(i,i) = -xx
  100 continue
 
  999 return
      end
c====================================================================
      subroutine t2ymlt(n,a,c)
 
c     c = aa@
c     where a is an upper triangular matrix and c is a symmetric matrix
c     whose lower triangle is calculated.
 
      implicit none
      integer n,i,j
      double precision a(n,n),c(n,n),prod
 
      do 100 i=1,n
         do 110 j=1,i
            call inprod(n+1-i,n,n,a(i,i),a(j,i),prod)
            c(i,j) = prod
  110    continue
  100 continue
 
      return
      end
