      character*16 function dfmt(x,skip,n,maxm,lf)
 
c     Character function that returns the "best" format to write out
c     a double precision number.  Depending on which format yields
c     the most significant digits, fixed format ("F") or scientific
c     notation ("1PE") is selected.  Arguments:
c        x    [in] double precision number to be written
c        skip [in] number of spaces to be skipped before writing x.
c                  If skip<0, an additional space is skipped if x>0 and n>0
c                  (to accommodate the minus sign in neighboring numbers).
c                  Since -0 cannot be distinguished from 0, special
c                  value -100 is used instead.
c        n    [in] number of spaces available for writing x (excluding
c                  additional spaces as specified by skip.)
c                  NOTE: if n<0, a attempt is made to use fewer than abs(n)
c                  spaces, namely by stripping leading blanks or trailing
c                  zeroes.  This is relevant only if x may be written in
c                  fixed format.  (If maxm>0, trailing zeroes are not
c                  affected; if n>0, trailing zeroes are replaced by
c                  leading blanks.)
c        maxm [in] maximum number of digits after the decimal point; only
c                  applicable if fixed format is best.  To disable such
c                  maximum, call with maxm=0.
c        lf   [in] integer controlling whether a linefeed is written after
c                  the number:  0=no;  other=yes.
c
c     Example:
c        double precision x
c        character dfmt*16
c        write(8,dfmt(x,-2,8,0,0))x
c
c     Note:  small n may result in ugly results if the number is too
c     large or small for the "f" format.  The "1pe" format needs 5
c     spaces for the decimal point and the exponent if x>0 (e.g., 1.2E-04),
c     and 6 spaces if x<0 (e.g., -1.2E-04).  So if x>0, make sure n>=6,
c     and if x<0, n>=7.  Failure to do so results in the fixed format,
c     which may yield 0. if abs(x)<1 and asterisks if abs(x)>1.
c
c     Note: if x is integer-valued, an attempt is made to use a fixed format
c     with one digit (0) after the decimal point.  While this is a waste
c     of space, omitting the zero would, in sentences, often be confusing.
c
c     Note: if x=NaN, fixed format is returned.  The write result
c     depends on compiler implementation of writing out NaN.
c
c     Note: rounding problem occurs in red Hat Linux GNU g77.
c     For example, dlog10(dble(10)) - 1 = -2.nnnE-17.
c
c     Constantijn Panis,  21 April 2001
c     Last change: account for g77
 
      implicit none
      integer n,skip,maxm,lf,ss,m,an,nsd,i,dlen,trm
      double precision x,ax,dn,tmp
      character i2c*12,fmt*16,ssc*3,anc*3,mc*2
 
      an=abs(n)
      dn=dble(an)
      ss=abs(skip)
      if(skip.eq.-100)ss=0
 
      if(x.ne.x)then
c        x is NaN
         if(n.lt.0)an=min(an,3)
         ssc=i2c(ss,0)
         anc=i2c(an,0)
c        fmt is something like '(4x,f8.0)'
         fmt='('//ssc(1:trm(ssc))//'x,f'//anc(1:trm(anc))//'.0)'
         goto 200
      endif
 
      ax=abs(x)
 
      if(x.ge.0.d0.and.skip.lt.0)then
c        For positive numbers, skip an additional space if requested
c        (less space left for number)
         ss=ss+1
         an=an-1
      endif
 
c     If number is integer-valued and small enough to fit in the
c     allotted space, write out as integer with ".0" attached.
c     This ".0" prevents confusion in reading such numbers; one
c     easily interprets "." as the end of a sentence.  There's one
c     exception in case there is insufficient space for ".0"; then
c     the "0" is supressed.
      if(x.eq.int(x).and.-10.d0**(dn-2).lt.x.and.x.lt.10.d0**(dn-1))then
         if(n.lt.0)an=min(an,dlen(x)+2)
c        In principle ".0", unless pressed for space:
         m=min(1,an-dlen(x)-1)
         anc=i2c(an,0)
         mc=i2c(m,0)
         if(ss.eq.0)then
c           fmt is something like '(f6.1)'
            fmt='(f'//anc(1:trm(anc))//'.'//mc(1:trm(mc))//')'
         else
c           Could add ss to an, but this would lead the program to
c           use the additional space in case of numbers that are
c           too large, instead of writing out asterisks.
            ssc=i2c(ss,0)
c           fmt is something like '(6x,f12.1)'
            fmt='('//ssc(1:trm(ssc))//'x,f'//anc(1:trm(anc))//'.'//
     *         mc(1:trm(mc))//')'
         endif
         goto 200
      endif
 
c     The number is not NaN and not (small enough) integer-valued.
c     Compute number of significant digits if fixed format were used.
c     No need to do so if there is insufficient space for scientific
c     notation; then fixed is always chosen.
      if((x.lt.0.d0.and.an.le.6).or.(x.ge.0.d0.and.an.le.5))then
c        Insufficient space for scientific notation.
         continue
      elseif(x-.5d0.le.-10.d0**(dn-2))then
c        The largest negative value that may be written out by the f format
c        is -10**(n-2), exclusive.  Note that x will be rounded.
         nsd=0
      elseif(x.le.-1.d0)then
c        Two spaces lost for sign and decimal point:
         nsd=an-2
c        But not all spaces may be utilized if maxm>0.
         if(maxm.gt.0)nsd=min(nsd,maxm+1+int(dlog10(dint(ax)+.5d0)))
      elseif(x.lt.0.d0)then
c        Spaces lost for sign, decimal point, and zeroes
         nsd=an-3+int(dlog10(dint(ax)+.5d0))
c        But not all spaces may be utilized if maxm>0.
         if(maxm.gt.0)nsd=max(0,maxm+int(dlog10(dint(ax)+.5d0)))
      elseif(x.lt.1.d0)then
c        Spaces lost for decimal point and zeroes
         nsd=an-2+int(dlog10(dint(ax)+.5d0))
c        But not all spaces may be utilized if maxm>0.
         if(maxm.gt.0)nsd=max(0,maxm+int(dlog10(dint(ax)+.5d0)))
      elseif(x+.5d0.le.10.d0**(dn-1))then
c        The largest positive value that may be written out by the f format
c        is 10**(n-1), exclusive.  Note that x will be rounded.
c        One space lost for decimal point:
         nsd=an-1
c        But not all spaces may be utilized if maxm>0.
         if(maxm.gt.0)nsd=min(nsd,maxm+1+int(dlog10(dint(ax)+.5d0)))
      else
c        Number is too large for fixed format.
         nsd=0
      endif
 
c     Scientific notation loses five spaces for the decimal point and
c     the exponent; if negative, one extra for the sign.  If there is
c     insufficient space for scientific notation, also select fixed
c     format, which will probably result in '****' upon output.
      if((x.lt.0.d0.and.(an.le.6.or.nsd.ge.an-6)).or.
     *   (x.ge.0.d0.and.(an.le.5.or.nsd.ge.an-5)))then
c        Fixed format:
         if(x.le.-1.d0)then
            m=an-3-int(dlog10(dint(ax)+.5d0))
         elseif(x.lt.0.d0)then
            m=an-3
         elseif(x.lt.1.d0)then
            m=an-2
         else
            m=an-2-int(dlog10(dint(ax)+.5d0))
         endif
         m=max(m,0)
         if(maxm.ne.0.and.m.gt.maxm)then
c           Number of digits behind the decimal point is limited;
c           overall width of number may be decreased, if desired.
            if(n.lt.0)an=an-(m-maxm)
            m=maxm
         else
c           Strip trailing zeroes (only if maxm=0):
            do 110 i=m,1,-1
               tmp=(ax-dint(ax))*10.d0**dble(i)
               if(abs(tmp-10.d0*dint(.1d0*tmp+.5d0)).gt..5d0)goto 120
  110       continue
c           Normally, trailing zeroes are replaced by leading blanks;
c           if desired, overall width of number is decreased instead.
  120       if(n.lt.0)an=an-(m-i)
            m=i
         endif
         anc=i2c(an,0)
         mc=i2c(m,0)
         if(ss.eq.0)then
c           fmt is something like '(f12.4)'
            fmt='(f'//anc(1:trm(anc))//'.'//mc(1:trm(mc))//')'
         else
c           Could add ss to an, but this would lead the program to
c           use the additional space in case of numbers that are
c           too large, instead of writing out asterisks.
            ssc=i2c(ss,0)
c           fmt is something like '(6x,f12.4)'
            fmt='('//ssc(1:trm(ssc))//'x,f'//anc(1:trm(anc))//'.'//
     *         mc(1:trm(mc))//')'
         endif
      else
c        Scientific notation
         if(x.lt.0.d0)then
            m=max(an-7,0)
         else
            m=max(an-6,0)
         endif
c        May add ss to an, because sufficient space is guaranteed
         an=an+ss
         anc=i2c(an,0)
         mc=i2c(m,0)
c        fmt is something like '(1pe12.4)'
         fmt='(1pe'//anc(1:trm(anc))//'.'//mc(1:trm(mc))//')'
      endif
 
c     If lf=0, linefeed must be suppressed:
  200 if(lf.eq.0)then
         dfmt=fmt(1:trm(fmt)-1)//',$)'
      else
         dfmt=fmt
      endif
 
  999 return
      end
 
 
