      subroutine fp2fpt(nfull,nfree,fp,fpt,fullbeta,fulltype,iffull)
 
c     Subroutine to transform first derivatives such that they
c     refer to transformed parameters.  Transformed parameters
c     are transforms of parameters such that their value is
c     restricted to a certain domain.
c
c     fp = first derivatives wrt true parameters
c     fpt = first derivatives wrt transformed parameters, i.e., the
c           parameters that are searched upon.
c     fullbeta = full vector of parameters, including non-estimated.
c     fulltype = integer array indicating restrictions placed on parameters,
c        and thus the type of transformation:
c     0: no restrictions;
c     1: parameter must be strictly positive.
c        This is insured through an exponent transformation.
c     2: parameter must be between -1 and 1, non-inclusive.
c        This is insured through a tangent transformation.
c     3: parameter must be between 0 and 1, non-inclusive.
c        This is insured through a tangent transformation.
c     41/4: parameter must be strictly larger than its predecessor.
c        This is insured through an exponent transformation of
c        the difference.  Note: the first of a series of 4's is
c        the base, i.e., T1 = t1
c                        T2 = t1 + exp(t2)
c                        T3 = t1 + exp(t2) + exp(t3)
c        where T1,T2,T3 are untransformed and t1,t2,t3 are searched upon.
c        The first of the series has fulltype=41; subsequent have 4.
c     5: part of parameters making up a covariance matrix, to be
c        searched using the Cholesky decomposition.  The Cholesky
c        decomposition requires that diagonal elements>0.
c
c     Note: fp and fpt may share memory space.
c
c     Constantijn Panis,  29 August 2002
c     Last change: iffull integer
 
      implicit none
      integer nfree,nfull,fulltype(nfull),iffull(nfull),
     *   i,j,n,intcontig
      double precision fp(nfree),fpt(nfree),fullbeta(nfull),
     *   deriv,dpvecsum,dxdxx
 
      j=0
      do 100 i=1,nfull
         if(iffull(i).eq.0)goto 100
         j=j+1
c        Derivative of parameter i wrt its transform:
         deriv = dxdxx(i,nfull,fullbeta,fulltype)
         if((fulltype(i).eq.41.or.fulltype(i).eq.4).and.i.lt.nfull)then
c           part of strictly increasing series (that are all estimated!):
            n = 1+intcontig(4,fulltype(i+1),nfull-i)
            fpt(j) = dpvecsum(fp(j),n)*deriv
         else
            fpt(j) = fp(j)*deriv
         endif
  100 continue
      return
      end
c=================================================================
      double precision function dxdxx(pos,nfull,fullbeta,fulltype)
 
c     Function to compute the derivative of a parameter wrt its
c     transform.
c
c     Constantijn Panis, 8 July 1995
 
      implicit none
      integer pos,nfull,fulltype(nfull)
      double precision fullbeta(nfull),tpar,pi,halfpi
      parameter(pi=3.141592653589793238462643d0,halfpi=.5d0*pi)
 
      if(fulltype(pos).eq.0)then
         dxdxx = 1.d0
      elseif(fulltype(pos).eq.1.or.fulltype(pos).eq.5)then
         dxdxx = fullbeta(pos)
      elseif(fulltype(pos).eq.2)then
         tpar = dtan(halfpi*fullbeta(pos))
         dxdxx = 1.d0/(halfpi*(1.d0+tpar*tpar))
      elseif(fulltype(pos).eq.3)then
         tpar = dtan(halfpi*fullbeta(pos)-halfpi)
         dxdxx = 1.d0/(pi*(1.d0+tpar*tpar))
      elseif(fulltype(pos).eq.41)then
         dxdxx = 1.d0
      elseif(fulltype(pos).eq.4)then
         dxdxx = fullbeta(pos)-fullbeta(pos-1)
      else
         goto 999
      endif
 
  999 return
      end
