      subroutine m3mult0034(task,nrowa,nrowb,numblockc,dimblockc,
     *   a,b,cblock,dsparse,factor)
 
c     Subroutine to compute A*B*C = D:
c
c            A(nrowa,nrowb)   B(nrowb,dimc)   C(dimc,dimc)   D(nrowa,dimc)
c
c     where dimc = numblockc*dimblockc
c
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     This routine assumes that C is symmetric and block-diagonal and
c     that D is sparse.
c
c     The structure of D (sparse) must be passed in, i.e., the first column
c     of dsparse must be filled in.
c
c     The four matrices may be in transposed format.  Integer task
c     may be 1000,0100,0010, et cetera, depending on which matrices
c     are transposed.  The first digit refers to A; the second to B;
c     the third to C; the fourth to D.  For example,
c        task = 1100 means that A and B are transposed: A'*B'*C = D
c        task =   10 means that C is transposed: A*B*C' = D
c     In this routine, only A and B may be transformed.  C is symmetric,
c     D must not be transformed.
c
c     Dimensions as given above assume no transpositions; if a matrix
c     is transposed, its dimensions really are reversed.
c
c     Integer factor determines:
c        factor = 0     D = A*B*C
c              else     D = D + factor*A*B*C
c
c     Constantijn Panis, 9 February 2002
 
      implicit none
      integer task,nrowa,nrowb,numblockc,dimblockc,factor,i,j,k,cold,
     *   ia,ib,ja,jb,jsa,jsb,dimc
      double precision a(nrowa*nrowb),b(nrowb*numblockc*dimblockc),
     *   cblock(dimblockc,dimblockc),dsparse(nrowa,2),abij
 
      dimc=numblockc*dimblockc
 
c     In its most basic form, D = D + A*B*C, the routine is very compact:
c
c     do 110 i=1,nrowa
c        do 120 j=1,dimc
c           abij = 0.d0
c           do 130 k=1,nrowb
c 130          abij = abij + a(i,k)*b(k,j)
c           do 140 l=1,dimc
c 140          d(i,l) = d(i,l) + abij*c(j,l)
c 120    continue
c 110 continue
c
c     The 130 loop computes element (i,j) of A*B, hence the name abij.
c     This element is computed with every element of row j of C and
c     added to D(i,.).
c
c     Complications arise because this routine allows for A, B, C and D to
c     be stored in transposed form.  Note that A and B are only used in the
c     130 loop, and C and D are only used in the 140 loop.  The positions
c     that are used are ia, ib, ic and id.  In the 130 and 140 loops, they
c     increase by ja, jb, jc and jd, where j stands for "jump".  If A, B,
c     C and D are not-transposed, these jumps are nrowa, 1, nrowc and nrowd,
c     respectively, where nrow stands for the number of rows.  If A is
c     transposed, this changes to 1; if B is transposed, it becomes nrowb,
c     et cetera.  The first time an element is accessed in the 130 and 140
c     loops, we need A(i,1), B(1,j), C(j,1) and D(i,1); these are coded
c     as sa, sb, sc and sd, where s stands for "start".  These start off at
c     1 in all four cases, the first time the 130/140 loops are used.
c     The next time they increase by 1, nrowb, 1 and 1; these are coded
c     as jsa, jsb, jsc and jsd, where js stands for "jump in start".
c     If a matrix is transposed, the jumps are different.
 
      if(mod(task/1000,10).eq.0)then
         ja=nrowa
         jsa=1
      else
         ja=1
         jsa=nrowb
      endif
      if(mod(task/100,10).eq.0)then
         jb=1
         jsb=nrowb
      else
         jb=dimc
         jsb=1
      endif
c     C is symmetric
c     D is sparse, must not be transformed.
      if(mod(task,10).ne.0)call bug(0,1)
 
      if(factor.eq.0)call zerodp(nrowa,dsparse(1,2))
      do 110 i=1,nrowa
c        Only one element of D(i,.) is relevant.
         cold=int(dsparse(i,1))
         if(cold.eq.0)goto 110
         do 120 j=1,dimc
c           D(i,cold)=+AB(i,j)*C(j,cold)
c           Only needed if C(j,cold)!=0
            if(int((j-1)/dimblockc).ne.int((cold-1)/dimblockc))goto 120
            abij = 0.d0
            ia=1+(i-1)*jsa
            ib=1+(j-1)*jsb
            do 130 k=1,nrowb
               abij = abij + a(ia)*b(ib)
               ia=ia+ja
               ib=ib+jb
  130       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            dsparse(i,2)=dsparse(i,2)+
     *         abij*cblock(mod(j-1,dimblockc)+1,mod(cold-1,dimblockc)+1)
  120    continue
  110 continue
 
      return
      end
 
