      subroutine nextbranch(level1,level2,branch,nbdim,nb,branchnum)
 
c     For many applications, the program needs to loop over all branches
c     and subbranches.  For example, suppose a probit outcome is at level
c     4.  For each level 2 branch, the program must then loop over all
c     possible level 3 and level 4 subbranches in order to cover all
c     level 4 outcomes.  This subroutine must then be called multiple
c     times;  with each call, the next branch tree is returned.
c     Arguments:
c        level1 [in] the level from which all subbranches need to be found
c        level2 [in] level at which all outcomes must be found
c        branch [in/out] current branch tree.  When first called, branch
c                    elements 2 through level1 must be assigned; elements
c                    level1+1 through level2 will be set to one (first
c                    subbranch).  Every subsequent call, branch elements
c                    level1+1 through level2 will be updated so that
c                    branch indicates the next branch tree.
c        nbdim  [in] dimension of nb
c        nb     [in] array with numbers of branches
c        branchnum [in/out] branch tree number.  To be called with branchnum=0
c                    the first time.  branchnum is incremented by one after
c                    each call.  When all branch trees have been covered,
c                    branchnum is returned as zero.
c
c     For example, suppose a probit outcome is at level 4.  For each
c     level 2 branch, the program must then loop over all possible level 3
c     and level 4 subbranches in order to cover all level 4 outcomes.
c     Suppose all subbranches off level 2 branch ib2 need to be found:
c        branch(2)=ib2
c        branchnum=0
c        call nextbranch(2,4,branch,nbdim,nb,branchnum)
c     will return branch=(ib2,1,1) and branchnum=1.  A subsequent
c        call nextbranch(2,4,branch,nbdim,nb,branchnum)
c     will return branch=(ib2,1,2) and branchnum=2, provided there are at least
c     two level 4 branches off the first level 3 branch of the ib2-th
c     level 2 branch.  Now suppose there are only two level 4 branches off
c     the first level 3 branch.  A subsequent
c        call nextbranch(2,4,branch,nbdim,nb,branchnum)
c     will return branch=(ib2,2,1) and branchnum=3.  Et cetera, until all
c     branch trees have been covered, so that branchnum=0 is returned.
c
c     Constantijn Panis, 15 July 1996
 
      implicit none
      integer level1,level2,branch(level2),nbdim,nb(nbdim,2),branchnum,
     *   ll,nmbrnch
 
      if(level2.lt.level1)goto 999
 
      if(branchnum.eq.0)then
         if(level2.eq.level1)then
c           Just one branch
            branch(level2)=1
         else
c           Take the first branch at every lower level, provided that
c           such subbranches exist.  First check whether any at all exist:
            if(nmbrnch(level1,branch,level2,nbdim,nb).eq.0)goto 999
            ll=level1
  100       ll=ll+1
            branch(ll)=0
  110       branch(ll)=branch(ll)+1
            if(branch(ll).gt.nmbrnch(ll-1,branch,ll,nbdim,nb))then
c              There are no ll+ branches off this branch
               ll=ll-1
               if(ll.eq.level1)goto 300
               goto 110
            endif
            if(ll.lt.level2)goto 100
         endif
         branchnum=1
         goto 999
      endif
 
      if(level2.eq.level1)goto 300
 
c     Find the next branch, starting at the lowest level.
      ll=level2
  200 branch(ll)=branch(ll)+1
      if(branch(ll).gt.nmbrnch(ll-1,branch,ll,nbdim,nb))then
c        Increment next higher level:
         ll=ll-1
         if(ll.eq.level1)goto 300
         goto 200
      elseif(ll.lt.level2)then
c        Start out at first of each subbranch, if it exists:
  210    ll=ll+1
         if(nmbrnch(ll-1,branch,ll,nbdim,nb).eq.0)then
c           There are no level2 branches off this branch
            ll=ll-1
            if(ll.eq.level1)goto 300
            goto 200
         endif
         branch(ll)=1
         if(ll.lt.level2)goto 210
      endif
 
      branchnum=branchnum+1
      goto 999
 
c     No more branch trees:
  300 branchnum=0
 
  999 return
      end
