      double precision function pnorm3(x1,x2,x3,rho12,rho13,rho23)
c
c     for the trivariate normal r.v. (x,y,w) with zero means, unit
c     variance and corr(x,y)=rho12,corr(y,w)rho23,corr(w,x)=rho13,tnor
c     uses the method of section 2 of steck (1958) to compute z= pr
c     (x<x1,y<x2,w<x3) accuracy of theresult depends on the
c     subprograms tfn and sfnb01. for more detail re accuracy see
c     d j daley (1974) applied stat., journal of the royal statistical
c     society (series c), vol. 23, pp 435-438.  for more detail re
c     trivariate normal see g.p. steck (1958) ann. math. stat.,
c     vol 29, pp 780-800.
c
c     owen's t-function is evaluated using simpson's rule integration.
c     steck's s-funtions is evaluated using a 16-point gaussian
c     quadrature.  i changed the name of function sfn to ssfn to
c     prevent possible interference with other s-functions
c     (e.g., survival functions).
c
c     If the correlation matrix (rho12,rho13,rho23) is not positive
c     definite, this function returns pnorm3=-1.
c
c     Adapted by Constantijn Panis, 20 December 1990:
c     if computed pnorm3<0, return 0.
c
c     Constantijn Panis, 27 November 1998
c     Last change: return -1 if not positive definite.
 
      implicit none
      double precision x1,x2,x3,rho12,rho13,rho23,x(6),xiv(3),xr(6),
     *   phix(6),r(6),y(6),yr(3),yrt(6),a(6),b(6),txa(6),sxab(6),
     *   det,del,tfn,ssfn,pnorm1
      integer i,j,ind
 
c     The following shortcut is not implemented because multiprobit
c     calculates derivatives wrt correlations numerically.  The
c     derivatives would then be less accurate.
c      if(rho12.eq.0.d0.and.rho13.eq.0.d0.and.rho23.eq.0.d0)then
c         pnorm3=pnorm1(x1)*pnorm1(x2)*pnorm1(x3)
c         goto 999
c      endif
 
      x(1)=x1
      x(2)=x2
      x(3)=x3
      do 8 i=1,3
         if(dabs(x(i)).lt.1.d-15)x(i)=1.d-15
    8 continue
      r(1)=rho12
      r(2)=rho23
      r(3)=rho13
      ind=0
 
c     ind=0 for all x'x of same sign, =1 otherwise. if x's are of
c     different signs, then permute to have x(3) of different sign
c     and change x(3),r(2), and r(3) signs.
 
      if(x(1).eq.0.d0)then
         if(x(2)*x(3).ge.0.d0)goto 10
      else
         if(x(1)*x(2).lt.0.d0)then
            if(x(1)*x(3).lt.0.d0)then
               call swap(x(1),x(3))
               call swap(r(1),r(2))
            else
               call swap(x(2),x(3))
               call swap(r(1),r(3))
            endif
         else
            if(x(1)*x(3).ge.0.d0)goto 10
         endif
      endif
      ind=1
      x(3)=-x(3)
      r(2)=-r(2)
      r(3)=-r(3)
c
c     check for positive definite correlation matrix.
c
   10 do 11 i=1,3
         y(i)=1.d0-r(i)*r(i)
   11 continue
      det=y(1)+y(2)+y(3)-2.d0+2.d0*r(1)*r(2)*r(3)
      if(det.le.1.d-15)goto 800
      det=1.d0/dsqrt(det)
      pnorm3=0.d0
      yrt(1)=1.d0/dsqrt(y(1))
      yrt(2)=1.d0/dsqrt(y(2))
      yrt(3)=1.d0/dsqrt(y(3))
      do 16 i=1,3
      j=i+3
      x(j)=x(i)
      xiv(i)=1.d0/x(i)
      phix(i)=pnorm1(x(i))
      phix(j)=phix(i)
      y(j)=y(i)
      yrt(j)=yrt(i)
      r(j)=r(i)
      yr(i)=r(i)*r(i+2)-r(i+1)
      xr(i)=x(i+1)-x(i)*r(i)
      if(dabs(xr(i)).gt.1.d-25)goto 14
      xr(i)=1.d-25
   14 xr(j)=x(i+2)-x(i)*r(i+2)
      if(dabs(xr(j)).ge.1.d-25)goto 15
      xr(j)=1.d-25
   15 a(i)=xr(i)*xiv(i)*yrt(i)
      a(j)=xr(j)*xiv(i)*yrt(i+2)
      b(i)=det*(yr(i)+y(i)*xr(j)/xr(i))
      b(j)=det*(yr(i)+y(i+2)*xr(i)/xr(j))
      pnorm3=pnorm3+(1.d0-del(a(i),a(j)))*phix(i)
   16 continue
      pnorm3=.5d0*pnorm3
      do 17 i=1,6
         txa(i)=tfn(x(i),a(i))
         sxab(i)=ssfn(x(i),a(i),b(i),phix(i),txa(i))
         pnorm3=pnorm3-.5d0*txa(i)-sxab(i)
   17 continue
      if(ind.ne.0)pnorm3=
     *   .5d0*(phix(1)+phix(5)-del(x(1),x(2)))-txa(1)-txa(5)-pnorm3
      if(pnorm3.lt.0.d0)pnorm3=0.d0
      goto 999
 
c     ERROR: correlation matrix not positive definite.
  800 pnorm3=-1.d0
      goto 999
 
  999 return
      end
c=====================================================================
      subroutine swap(x,y)
      implicit none
      double precision x,y,z
 
      z=x
      x=y
      y=z
 
      return
      end
c=====================================================================
      double precision function del(x,y)
      implicit none
      double precision x,y
 
      if(x*y.lt.0.d0)then
         del=1.d0
      elseif(x*y.eq.0.d0)then
         if(x+y.lt.0.d0)then
            del=1.d0
         else
            del=0.d0
         endif
      else
         del=0.d0
      endif
 
      return
      end
c=====================================================================
      double precision function tfn(z,a)
 
c     Owens t function accurate to at least 1.d-10
 
      implicit none
      double precision z,a,x1,a1,ff1,f1,gg1,gg2,simpsn,pnorm1
 
      x1=z
      a1=a
      ff1=0.d0
      f1=1.d0
      tfn=0.d0
      if(a.ge.0.d0)goto 10
      a1=-a1
      f1=-1.d0
   10 if(a1.le.1.d0)goto 15
      x1=dabs(x1)
      gg1=pnorm1(x1)
      x1=a1*x1
      a1=1.d0/a1
      gg2=pnorm1(x1)
      ff1=.5d0*(gg1+gg2)-gg1*gg2
      ff1=ff1*f1
      f1=-f1
   15 tfn=ff1 + f1*simpsn(0.d0,a1,x1)
 
      return
      end
c=====================================================================
      double precision function tigrnd(a,h)
      implicit none
 
c     integrand of owen's t-function
c     (ref: owen, d b, ann. math. stat., 27, 1956, pp 1075-1090)
 
      double precision a,h,aa,hh
 
      aa=1.d0+a*a
      hh=h*h
      tigrnd=.1591549431d0*dexp(-.5d0*hh*aa)/aa
 
      return
      end
c=====================================================================
      double precision function ssfn(x,a,b,phix,txa)
 
c     computes steck's (1958 ann math stat) s-fn with max error e-9,
c     though computer rounding errors may reduce this accuracy. calls
c     function routines sfnb01(x,a,b) in which abs(b) <= 1, pnorm1(x),
c     and tfn(x,a).
c     this function used to be called sfn, but this may interfere
c     with other s-functions (e.g., survival functions).
c     Renamed by Constantijn Panis,  November 1988
 
      implicit none
      double precision x,a,b,phix,txa,ba,xa,sfnb01,ta,aa,xab,biv,xf,
     *   ab,tfn,phixab,pnorm1,xfa
 
      ba = dabs(b)
      xa = dabs(x)
      if(ba.gt.1.d0)goto 1
      ssfn=sfnb01(xa,a,ba)
      goto 3
    1 ta=dabs(txa)
      aa=dabs(a)
      biv=1.d0/ba
      xf=dabs(phix-.5d0)
      xab=xa*aa*ba
      phixab=pnorm1(xab)
      xfa=phixab-.5d0
      if(aa.le.1.d0)goto 2
      ssfn=xfa*ta - xf*tfn(xab,biv) + .25d0*(xf-xfa) +
     *    sfnb01(xab,biv,1.d0/aa)
      goto 3
    2 ab=aa*ba
      ssfn=.25d0*(xf+.5d0) + xfa*ta - sfnb01(xab,1.d0/ab,aa) -
     *    sfnb01(xa,ab,biv)
    3 if(x.ge.0.d0)goto 4
      ssfn=.1591549431d0*datan(ba/dsqrt(1.d0+a*a*(1.d0+b*b))) - ssfn
    4 if(b.ge.0.d0)goto 999
      ssfn=-ssfn
  999 return
      end
c=====================================================================
      double precision function sfnb01(x,a,b)
 
c     function routine which evaluates steck's s-function via a
c     16-point legendre-gauss quadrature.
c
c     references:  steck, g p, ann. math. stat., 29, 1958, pp 780-800.
c                  daley, d j, applied stat., journal of the royal
c                              statistical society (series c), 23,
c                              1974, pp 435-438.
c                  abramowitz, m, & stegun, i, handbook of math.
c                              functions, 1964, p 916.
c                  kennedy, w j, & gentle, j e, statistical computing,
c                              1980, pp 80-86.
 
      implicit none
      double precision x,a,b,wgt(8),abscis(8),sigrnd
      integer i
      data       wgt/.189450610455068496285d0,
     *               .182603415044923588867d0,
     *               .169156519395002538189d0,
     *               .149595988816576732081d0,
     *               .124628971255533872052d0,
     *               .095158511682492784810d0,
     *               .062253523938647892863d0,
     *               .027152459411754094852d0/
      data    abscis/.095012509837637440185d0,
     *               .281603550779258913230d0,
     *               .458016777657227386342d0,
     *               .617876244402643748447d0,
     *               .755404408355003033895d0,
     *               .865631202387831743880d0,
     *               .944575023073232576078d0,
     *               .989400934991649932596d0/
 
      sfnb01=0.d0
      do 100 i=1,8
         sfnb01=sfnb01 + wgt(i)*(sigrnd(x,a,b, abscis(i)) +
     *                           sigrnd(x,a,b,-abscis(i)))
  100 continue
 
      return
      end
c=====================================================================
      double precision function sigrnd(x,a,b,y)
 
c     function routine which defines the integrand for steck's
c     s-function (eqn 3.7: steck, g p, ann. math. stat., 29, 1958,
c     pp 780-800).  note that the range of the integral has been
c     transformed from (0,1) to (-1,1) to conform with the range of a
c     legendre-gauss quadrature.
 
      implicit none
      double precision x,a,b,y,yy,bysq,abysq,fourpi,pnorm1
      data  fourpi/.7957747155d-1/
 
      yy=.5d0*y + .5d0
      bysq=1.d0 + b*b*yy*yy
      abysq=dsqrt(1.d0 + a*a*bysq)
 
      sigrnd=fourpi*b*pnorm1(x*abysq)/(bysq*abysq)
 
      return
      end
c=====================================================================
      double precision function simpsn(a,b,e1)
 
c      simpsn integrates given fortran function by simpson's rule,
c   successively using shorter and shorter intervals until convergence
c   is achieved.
c
c   calling sequence:   ...simpsn(a,b,e1)...
c
c                       a,b   = limits of integration
c
c   simpsn selects an interval size, computes the area under fn using
c   simpson's rule, bisects these intervals, computes the areas again
c   with the smaller intervals, and continues until one of the
c   convergence criteria is satisfied.
c
c   convergence criteria:   mlimit = maximum number of bisections
c                           conv   = maximum change in tigrnd function
c
c     Constantijn Panis, 29 March 1996
c     Change: external function passed as argument now hardwired to
c        be tigrnd.
 
      implicit none
      double precision a,b,tigrnd,e1,fa,fb,h,c(2),s(2,2),conv
      integer i,m,mlimit
      data conv/1.d-6/
      data mlimit/256/
 
      m=8
      fa=tigrnd(a,e1)
      fb=tigrnd(b,e1)
      h=(b-a)/8.d0
      s(1,1)=tigrnd(a+2.d0*h,e1)+tigrnd(a+4.d0*h,e1)+tigrnd(a+6.d0*h,e1)
      s(1,2)=tigrnd(a+h,e1) + tigrnd(a+3.d0*h,e1) +
     *   tigrnd(a+5.d0*h,e1) + tigrnd(a+7.d0*h,e1)
      c(1)=h*(fa+fb+2.d0*s(1,1)+4.d0*s(1,2))/3.d0
 
c     Successively integrate tigrnd using shorter and shorter intervals
c     until convergence is achieved.
   20 m=2*m
      h=h*.5d0
      s(2,1)=s(1,1)+s(1,2)
      s(2,2)=0.d0
      do 25 i=2,m,2
         s(2,2)=s(2,2)+tigrnd(a+(i-1)*h,e1)
   25 continue
      c(2)=h*(fa+fb+2.d0*s(2,1)+4.d0*s(2,2))/3.d0
      if(dabs(c(1)-c(2))-conv.lt.0.d0)goto 65
      if (m-mlimit.gt.0.d0)goto 65
      c(1)=c(2)
      s(1,1)=s(2,1)
      s(1,2)=s(2,2)
      goto 20
 
c     Convergence achieved
   65 simpsn=c(2)+(c(2)-c(1))/15.d0
 
      return
      end
