      subroutine spline(nds,ni,refspl,nodepos,numnodes,spltype,nngam,
     *   time,gamma,s,ifdrv,dsdgamma,xnode,origin,ipt,jpt)
 
c     Subroutine to compute the log of an overlapping spline survivor
c     function.
c     The number of overlaps is nds: arbitrary up to a maximum of
c     maxnds.  The log-hazard function is assumed to be the sum of
c     nds spline hazards; they are superimposed to express the idea
c     that the hazard process is driven by nds time notions.
c     The number of nodes for each process is arbitrary up to a maximum
c     of maxnode.
c     It is assumed that each spline duration starts "origin" before the
c     beginning of the episode.  For example, a duration spline for the
c     respondent's age has origin=20 if the respondent is 20 years old
c     at the beginning of the episode.
c
c     The arguments are as follows:
c     nds    [in]  number of duration splines
c     it     [in]  number of time points
c     refspl [in]  spline definition numbers
c     nodepos [in] begin position in xnode of spline nodes
c     numnodes [in] numbers of nodes
c     spltype [in] spline type: 1 = no intercept, full effect
c                               2 = intercept, full effect
c                               3 = no intercept, right effect
c                               4 = intercept, right effect
c     nngam  [in]  total number of gammas (intercepts plus slopes)
c     time   [in]  time points at which survival function needs to be
c                  evaluated
c     gamma  [in]  nds sets of gammas, stored in one-dimensional array
c     s      [out] survival function values
c     ifdrv  [in]  are derivatives desired?
c     dsdgamma [out] derivatives of s wrt gamma
c     xnode  [in]  array with spline nodes and other things
c     origin [in]  nds periods that episode starts after the spline origin.
c                  If origin>0, the spline is already active; if origin<0,
c                  the spline's origin is somewhere after the beginning
c                  of the episode.
c
c     Note: this subroutine aborts program execution if:
c        maxnds (maximum number of duration splines) is exceeded
c        maxnode (maximum number of nodes) is exceeded
c        maxgam (maximum number of gammas) is exceeded
c        nodes are unordered
c        the first time mark preceeds the earliest process.
c
c     Constantijn Panis,  13 November 1997
c     Last change: compute log(S), not S
 
      implicit none
      integer nds,ni,nngam,maxnode,maxgam,maxnds
      parameter(maxnds=40,maxnode=100,maxgam=maxnode+maxnds)
      double precision gamma(nngam),s(0:ni),dsdgamma(0:ni,nngam),
     *   area(maxnode),intercept(maxnode),slope(maxnode),
     *   darea(maxgam*maxnode),dintercept(maxgam*maxnode),
     *   dslope(maxgam*maxnode)
      real xnode(*),outnode(maxnode),time(0:ni),origin(nds)
      integer refspl(nds),nodepos(*),numnodes(*),spltype(*),ipt,jpt,
     *   nnodes
      logical ifdrv
 
      if(nds.gt.maxnds.or.nngam-nds.gt.maxnode)call bug(ipt,jpt)
 
c     The approach is as follows.  First all nodes in all duration splines
c     are ordered.
c     The ordered set of nodes is labeled outnode.  The lowest outnode
c     is zero.
c     Each outnode has three numbers attached to it: area, intercept and
c     slope.  In understanding these three numbers it is good to have a
c     picture of a spline hazard function in mind.  Recall that the
c     survival function is the exponent of minus the integral over the
c     hazard function from the begin point of the hazard function to
c     the time mark.  This integral is an area, which can be split up
c     into the area up until the highest node that is still lower than
c     the time mark and the area from that node up until the time mark.
c     Variable area(i) is the cumulative area of the hazard function
c     (not the log-hazard function) up until outnode(i), i.e., it
c     provides the first area in the integral.  Variable intercept(i)
c     is the intercept of the log of the hazard function at outnode(i);
c     variable slope(i) is the slope of the log of the hazard function
c     at outnode(i).  Recall that the log of the hazard function is
c     a linear spline, so the slope is constant between every pair of
c     nodes.  Variables intercept(i) and slope(i) contain sufficient
c     information to compute the second part of the integral, the area
c     between outnode(i) and the time mark.
c     If derivatives are desired, the derivatives of area, intercept
c     and slope with respect to all gammas are also computed and stored
c     in darea, dintercept and dslope.
      call order(nds,xnode,outnode,gamma,area,intercept,
     *   slope,nngam,nnodes,origin,time(ni),maxnode,refspl,nodepos,
     *   numnodes,spltype,ifdrv,darea,dintercept,dslope,ipt,jpt)
 
c     The nodes are ordered and arrays area, intercept and slope are
c     computed.  The overlapping spline has been converted in a simple
c     spline; compute the survival function and its derivatives with
c     respect to the gammas.
      call onespline(time,ni,outnode,nnodes,area,intercept,slope,s,
     *   ifdrv,dsdgamma,darea,dintercept,dslope,nngam,ipt,jpt)
 
  999 return
      end
c=====================================================================
      subroutine order(nds,xnode,outnode,gamma,area,intercept,
     *   slope,nngam,nnodes,origin,tmax,maxnode,refspl,nodepos,
     *   numnodes,spltype,ifdrv,darea,dintercept,dslope,ipt,jpt)
 
c     Subroutine to order nodes for overlapping spline and to compute
c     three numbers for each ordered node: the area under the hazard
c     function from the beginning of the spline until that node, the
c     intercept of the log-hazard function at that node and the slope
c     of the log-hazard function at that node.  These three numbers
c     strongly facilitate the computation of the spline survival
c     function.
c
c     There are nds overlapping duration splines, each starting origin(.)
c     before some agreed upon zero point.  (The zero point is the time
c     point at which the duration of an episode starts.)
c
c     Arguments:
c     nds        [in]  number of overlapping duration splines
c     xnode      [in]  array with spline nodes and other things
c     outnode    [out] output nodes, ordered.
c     gamma      [in]  nds sets of input gammas
c     area       [out] area under hazard function from beginning of time
c                      until a particular output node.
c     intercept  [out] intercept of log-hazard function at a particular
c                      output node.
c     slope      [out] slope of log-hazard function at a particular
c                      output node.
c     nngam      [in]  total number of gammas (intercepts and slopes)
c     typespl    [in]  type of spline: 1 = no intercept, 2 = intercept
c     nnodes     [out] number of output nodes
c     origin     [in]  time lags for each of the nds duration splines.
c                      Note that a positive origin implies that the spline
c                      starts before time=0; negative origin implies that
c                      the spline starts after time=0.
c     tmax       [in]  highest time mark; allows gaining efficiency.
c                      Must be set for proper functioning.
c     maxnode    [in]  maximum number of nodes allowed for any of the
c                      input splines and for the ordered (output) spline
c     refspl     [in]  spline definition numbers
c     nodepos    [in]  begin position in xnode of spline nodes
c     numnodes   [in]  numbers of nodes
c     spltype    [in] spline type: 1 = no intercept, full effect
c                                  2 = intercept, full effect
c                                  3 = no intercept, right effect
c                                  4 = intercept, right effect
c     ifdrv      [in]  want derivatives?
c     darea      [out] derivatives of area wrt gamma
c     dintercept [out] derivatives of intercept wrt gamma
c     dslope     [out] derivatives of slope wrt gamma
 
      implicit none
      integer nds,maxnode,nngam,nnodes
      double precision gamma(nngam),area(maxnode),intercept(maxnode),
     *   slope(maxnode),darea(nngam,maxnode),dintercept(nngam,maxnode),
     *   dslope(nngam,maxnode),temp,temp1,temp2
      real xnode(*),outnode(maxnode),origin(nds),tmax,oldnode,node,diff
      integer refspl(nds),nodepos(*),numnodes(*),spltype(*),ipt,jpt,
     *   ref,old,ds,i,j,ln,jn,jg,jjn,jjg,match
      logical ifdrv,splok,realsordered
 
c     First determine what the output nodes will be.  Check that there
c     is at least one spline operative at the beginning of the episode.
      splok=.false.
      outnode(1)=0.
      nnodes=1
      do 100 ds=1,nds
         ref=refspl(ds)
         if(.not.realsordered(xnode(nodepos(ref)),numnodes(ref)))
     *      goto 800
         splok=(splok.or.spltype(ref).le.2.or.origin(ds).ge.0.)
         if(origin(ds).lt.0..and.-origin(ds).lt.tmax.and.spltype(ref)
     *      .ge.3)call insertnode(-origin(ds),outnode,nnodes)
         do 110 i=0,numnodes(ref)-1
            node=xnode(nodepos(ref)+i)-origin(ds)
            if(node.ge.tmax)then
               goto 100
            elseif(node.gt.0.)then
               call insertnode(node,outnode,nnodes)
            endif
  110    continue
  100 continue
      if(.not.splok)goto 800
 
      call zerodp(nnodes,area)
      call zerodp(nnodes,intercept)
      call zerodp(nnodes,slope)
      if(ifdrv)then
         call zerodp(nngam*nnodes,darea)
         call zerodp(nngam*nnodes,dintercept)
         call zerodp(nngam*nnodes,dslope)
      endif
 
c     Loop over all splines and determine intercepts and slopes:
      jg=1
      do 200 ds=1,nds
         ref=refspl(ds)
         jn=nodepos(ref)
         ln=jn+numnodes(ref)-1
 
c        Accumulate defined intercept, unless origin>0 and effect is right only.
         if(spltype(ref).eq.2.or.
     *      (spltype(ref).eq.4.and.origin(ds).ge.0.))then
            intercept(1)=intercept(1)+gamma(jg)
            if(ifdrv)dintercept(jg,1)=1.d0
            jg=jg+1
         endif
 
         if(origin(ds).gt.0.)then
c           Accumulate intercept at beginning of episode.
c           Loop over nonpositive nodes, if any:
            call gotoposnode(jn,jg,ln,xnode)
            oldnode=-origin(ds)
  210       node=xnode(jn)-origin(ds)
            if(jn.le.ln.and.node.lt.0.)then
               diff=node-oldnode
               intercept(1)=intercept(1)+gamma(jg)*dble(diff)
               if(ifdrv)dintercept(jg,1)=dble(diff)
               jg=jg+1
               jn=jn+1
               oldnode=node
               goto 210
            endif
c           Either there are no more nodes, or subsequent nodes are
c           at or after the beginning of the episode:
            intercept(1)=intercept(1)-gamma(jg)*dble(oldnode)
            if(ifdrv)dintercept(jg,1)=-dble(oldnode)
            if(jn.le.ln.and.node.eq.0.)then
               jg=jg+1
               jn=jn+1
            endif
         elseif(origin(ds).eq.0.)then
c           Loop over nonpositive nodes, if any:
            call gotoposnode(jn,jg,ln,xnode)
         else
c           Spline origin is after the beginning of the episode.
c           If this spline kicks in, no problem.  If its effect
c           is full, however, compute the intercept at the beginning
c           of the episode.
            if(spltype(ref).ge.3)goto 300
  220       if(jn.gt.ln)then
c              No nodes between the beginning of the episode and origin:
               intercept(1)=intercept(1)+gamma(jg)*dble(origin(ds))
               if(ifdrv)dintercept(jg,1)=dble(origin(ds))
               goto 300
            endif
c           Node is relative to the start of the episode:
            node=xnode(jn)-origin(ds)
            if(node.lt.0.)then
c              Irrelevant node: skip it and try next node
               jn=jn+1
               jg=jg+1
               goto 220
            elseif(node.eq.0.)then
c              Node happens to coincide with start of episode: skip it and
c              position jn/jg to first relevant node/slope pair.
               jn=jn+1
               jg=jg+1
               jjg=jg
               jjn=jn
            elseif(node.gt.0.)then
c              Node is after the start of the episode, so jn/jg are positioned
c              correctly.  Account for the period between the beginning of the
c              episode and the first nonnegative node or origin, whichever
c              comes first:
               diff=min(node,-origin(ds))
               intercept(1)=intercept(1)-gamma(jg)*dble(diff)
               if(ifdrv)dintercept(jg,1)=-dble(diff)
c              Only continue accumulating if the node is between the start of
c              the episode and the origin:
               if(xnode(jn).ge.0.)goto 300
               jjg=jg+1
               jjn=jn+1
            endif
c           Account for period between first nonnegative node and origin:
  230       if(jjn.gt.ln.or.xnode(jjn).ge.0.)then
               oldnode = xnode(jjn-1)
               intercept(1)=intercept(1)+gamma(jjg)*dble(oldnode)
               if(ifdrv)dintercept(jjg,1)=dble(oldnode)
               goto 300
            elseif(xnode(jjn).lt.0.)then
               diff=xnode(jjn)-xnode(jjn-1)
               intercept(1)=intercept(1)-gamma(jjg)*dble(diff)
               if(ifdrv)dintercept(jjg,1)=-dble(diff)
               jjg=jjg+1
               jjn=jjn+1
               goto 230
            endif
         endif
 
  300    if(spltype(ref).ge.3.and.origin(ds).lt.0.)then
c           Spline has yet to kick in; set 'm up:
            call realmatch(-origin(ds),outnode,nnodes,old)
            if(old.eq.0)goto 800
            if(spltype(ref).eq.4)then
               intercept(old)=intercept(old)+gamma(jg)
               if(ifdrv)dintercept(jg,old)=1.d0
               jg=jg+1
            endif
         else
c           Spline has full effect or has already kicked in.
            old=1
         endif
 
c        Go over all nodes of this spline during the episode and determine
c        the slopes; intercepts are computed off of slopes in the 400 loop.
  310    node=xnode(jn)-origin(ds)
         if(jn.gt.ln.or.node.ge.tmax)then
c           No more nodes before tmax; wrap this spline up.
            do 320 j=old,nnodes
               slope(j)=slope(j)+gamma(jg)
               if(ifdrv)dslope(jg,j)=1.d0
  320       continue
            jg=jg+1
            if(jn.le.ln)jg=jg+ln-jn+1
            goto 200
         endif
         if(node.le.0.)goto 800
c        Accumulate slope up until the node:
         call realmatch(node,outnode(old),nnodes-old+1,match)
         if(match.eq.0)goto 800
         call sadd(slope(old),gamma(jg),slope(old),match-1)
         if(ifdrv)call putrow(1.d0,dslope(jg,old),match-1,nngam)
         old=old+match-1
         jn=jn+1
         jg=jg+1
         goto 310
c        NOTE: the following 330 loop is identical to the above but caused
c        inexplicable error in release version of Digital Fortran (not in
c        debug version, i.e., probably due to improper optimization.)
c         do 330 j=old,nnodes
c            if(node.eq.outnode(j))then
c               old=j
c               jn=jn+1
c               jg=jg+1
c               goto 310
c            endif
c            slope(j)=slope(j)+gamma(jg)
c            if(ifdrv)dslope(jg,j)=1.d0
c  330    continue
c         goto 800
  200 continue
 
c     Add intercept at previous node to intercept and compute area:
      do 400 i=2,nnodes
         diff = outnode(i)-outnode(i-1)
         if(slope(i-1).eq.0.d0)then
            intercept(i)=intercept(i)+intercept(i-1)
            temp = dexp(intercept(i-1))*dble(diff)
            area(i) = area(i-1) + temp
            if(ifdrv)then
               do 420 j=1,nngam
                  dintercept(j,i)=dintercept(j,i)+dintercept(j,i-1)
                  darea(j,i)=darea(j,i-1)+temp*dintercept(j,i-1)
  420          continue
            endif
         else
            intercept(i)=intercept(i)+intercept(i-1)+
     *         slope(i-1)*dble(diff)
            if(ifdrv)then
               do 430 j=1,nngam
                  dintercept(j,i)=dintercept(j,i)+dintercept(j,i-1)+
     *               dslope(j,i-1)*dble(diff)
  430          continue
            endif
c           temp1 is the exponent of the new intercept, but without possible
c           jumps caused by splines starting at the new node; temp2 is
c           the exponent of the old intercept.
            temp1 = dexp(intercept(i-1) + slope(i-1)*dble(diff))
            temp2 = dexp(intercept(i-1))
            area(i) = area(i-1) + (temp1-temp2)/slope(i-1)
            if(ifdrv)then
               do 440 j=1,nngam
                  darea(j,i)=darea(j,i-1)+
     *            (temp1*(dintercept(j,i-1)+dslope(j,i-1)*dble(diff)) -
     *             temp2*dintercept(j,i-1) -
     *             (temp1-temp2)*dslope(j,i-1)/slope(i-1))/slope(i-1)
  440          continue
            endif
         endif
  400 continue
 
  999 return
  800 call bug(ipt,jpt)
      end
c=====================================================================
      subroutine onespline(time,ni,node,nnode,area,intercept,slope,s,
     *   ifdrv,dsdgamma,darea,dintercept,dslope,nngam,ipt,jpt)
 
c     The survival function is easily computed using area, intercept
c     and slope.  Also available as inputs are darea, dintercept and
c     dslope, the derivatives of area, intercept and slope with respect to
c     all gammas.  Derivatives of the survival function with respect to
c     all gammas follows from the chain rule: derivative wrt gamma =
c     derivative of survival function wrt area times derivative of area
c     wrt gamma plus derivative of survival function wrt intercept
c     times derivative of intercept wrt gamma plus derivative of
c     survival function wrt slope times derivative of slope wrt gamma.
c
c     Constantijn Panis, 1 July 1993
 
      implicit none
      integer ni,nnode,nngam,ipt,jpt,inode,i,j
      double precision area(nnode),intercept(nnode),slope(nnode),
     *   s(0:ni),temp,temp1,temp2,temp3,dsdgamma(0:ni,nngam),
     *   darea(nngam,nnode),dintercept(nngam,nnode),dslope(nngam,nnode)
      real node(nnode),time(0:ni),diff
      logical ifdrv
 
      if(time(1).lt.node(1))call bug(ipt,jpt)
      inode=1
 
      do 100 i=0,ni
         if(inode.gt.nnode)goto 120
  110    if(node(inode).lt.time(i))then
c           node is less than time mark; go up one node to see whether
c           that node also is less than time mark, unless there is no
c           more node.
            inode=inode+1
            if(inode.gt.nnode)goto 120
            goto 110
         elseif(node(inode).eq.time(i))then
c           node is equal to time mark
            s(i) = -area(inode)
            if(ifdrv)then
               do 210 j=1,nngam
                  dsdgamma(i,j)=-darea(j,inode)
  210          continue
            endif
c           next time mark is at least as large; speed up the search
            inode=inode+1
            goto 100
         endif
c        node is too large; back up one and compute spline
  120    inode=inode-1
         diff = time(i)-node(inode)
         if(slope(inode).eq.0.d0)then
            temp = dble(diff)*dexp(intercept(inode))
            s(i) = -(area(inode)+temp)
            if(ifdrv)then
               do 220 j=1,nngam
                  dsdgamma(i,j) =
     *            -(darea(j,inode) + dintercept(j,inode)*temp +
     *             dslope(j,inode)*.5d0*dble(diff)*temp)
  220          continue
            endif
         else
            temp1 = dexp(intercept(inode)+slope(inode)*dble(diff))
            temp2 = dexp(intercept(inode))
            temp3 = (temp1-temp2)/slope(inode)
            s(i) = -(area(inode)+temp3)
            if(ifdrv)then
               do 230 j=1,nngam
                  dsdgamma(i,j) =
     *            -(darea(j,inode) + dintercept(j,inode)*temp3 +
     *            dslope(j,inode)*(dble(diff)*temp1-temp3)/slope(inode))
  230          continue
            endif
         endif
c        next time mark is at least as large; speed up the search
         inode=inode+1
  100 continue
 
  999 return
      end
c=====================================================================
      subroutine insertnode(node,outnode,nnodes)
 
c     Subroutine to insert a node and update the number of nodes.
c     Outnode has dimension nnodes, but since nnodes may be changed
c     in this routine, outnode(*) is declared.
c
c     Constantijn Panis, 21 August 1993
 
      implicit none
      integer nnodes,i,j
      real node,outnode(*)
 
      do 100 i=1,nnodes
         if(node.eq.outnode(i))goto 999
         if(node.lt.outnode(i))then
            do 110 j=nnodes,i,-1
               outnode(j+1)=outnode(j)
  110       continue
            outnode(i)=node
            nnodes=nnodes+1
            goto 999
         endif
  100 continue
      nnodes=nnodes+1
      outnode(nnodes)=node
 
  999 return
      end
c========================================================================
      subroutine gotoposnode(jn,jg,ln,xnode)
 
c     Subroutine to skip over nodes that were defined as negative or zero,
c     and to position jn and jg at the first strictly zero node with the
c     gamma just preceding that node.
c
c     Constantijn Panis, 21 August 1993
 
      implicit none
      integer jn,jg,ln
      real xnode(*)
 
  100 if(jn.le.ln.and.xnode(jn).le.0.)then
         jn=jn+1
         jg=jg+1
         goto 100
      endif
 
      return
      end
